[dpdk-dev] i40evf: fix link info update

Message ID 1459474604-2049-1-git-send-email-jingjing.wu@intel.com (mailing list archive)
State Changes Requested, archived
Headers

Commit Message

Jingjing Wu April 1, 2016, 1:36 a.m. UTC
  The issue is the VF's link speed kept as 10G and status always was up.
It did not change even the physical link's status changed.
This patch fixes this issue to make VF's link info consistent with
physical link.

Fixes: 4861cde46116 (i40e: new poll mode driver)
Signed-off-by: Jingjing Wu <jingjing.wu@intel.com>
---
 doc/guides/rel_notes/release_16_04.rst |  5 +++++
 drivers/net/i40e/i40e_ethdev.h         |  1 +
 drivers/net/i40e/i40e_ethdev_vf.c      | 33 +++++++++++++++++++++++++++++----
 3 files changed, 35 insertions(+), 4 deletions(-)
  

Comments

Thomas Monjalon April 1, 2016, 8:23 a.m. UTC | #1
Hi Jingjing,

Please rebase on top of Marc's patches.

2016-04-01 09:36, Jingjing Wu:
> +* **i40e: Fixed link info of VF

a blank line is missing

> +  Previously, the VF's link speed kept as 10G and status always was up. It did not
> +  change even the physical link's status changed. Now this issue is fixed to make
> +  VF's link info consistent with physical link.

[...]
> +			new_link.link_speed = ETH_LINK_SPEED_100;
> +			break;
> +		case I40E_LINK_SPEED_1GB:
> +			new_link.link_speed = ETH_LINK_SPEED_1000;

These speeds have new names after Marc's rework.

> +		new_link.link_status = vf->link_up ? 1 : 0;

Please use new constants for link up and down.
  

Patch

diff --git a/doc/guides/rel_notes/release_16_04.rst b/doc/guides/rel_notes/release_16_04.rst
index 79d76e1..b44a02f 100644
--- a/doc/guides/rel_notes/release_16_04.rst
+++ b/doc/guides/rel_notes/release_16_04.rst
@@ -366,6 +366,11 @@  Drivers
   info for l3fwd to work well. Now there is an option for l3fwd to analysis
   packet type softly, so enable vector driver by default.
 
+* **i40e: Fixed link info of VF
+  Previously, the VF's link speed kept as 10G and status always was up. It did not
+  change even the physical link's status changed. Now this issue is fixed to make
+  VF's link info consistent with physical link.
+
 * **mlx5: Fixed possible crash during initialization.**
 
   A crash could occur when failing to allocate private device context.
diff --git a/drivers/net/i40e/i40e_ethdev.h b/drivers/net/i40e/i40e_ethdev.h
index ce945fe..cfd2399 100644
--- a/drivers/net/i40e/i40e_ethdev.h
+++ b/drivers/net/i40e/i40e_ethdev.h
@@ -502,6 +502,7 @@  struct i40e_vf {
 	/* Event from pf */
 	bool dev_closed;
 	bool link_up;
+	enum i40e_aq_link_speed link_speed;
 	bool vf_reset;
 	volatile uint32_t pend_cmd; /* pending command not finished yet */
 	uint32_t cmd_retval; /* return value of the cmd response from PF */
diff --git a/drivers/net/i40e/i40e_ethdev_vf.c b/drivers/net/i40e/i40e_ethdev_vf.c
index a13d9cc..71af0f7 100644
--- a/drivers/net/i40e/i40e_ethdev_vf.c
+++ b/drivers/net/i40e/i40e_ethdev_vf.c
@@ -258,6 +258,8 @@  i40evf_read_pfmsg(struct rte_eth_dev *dev, struct i40evf_arq_msg_info *data)
 		case I40E_VIRTCHNL_EVENT_LINK_CHANGE:
 			vf->link_up =
 				vpe->event_data.link_event.link_status;
+			vf->link_speed =
+				vpe->event_data.link_event.link_speed;
 			vf->pend_msg |= PFMSG_LINK_CHANGE;
 			PMD_DRV_LOG(INFO, "Link status update:%s",
 				    vf->link_up ? "up" : "down");
@@ -1310,6 +1312,7 @@  i40evf_handle_pf_event(__rte_unused struct rte_eth_dev *dev,
 {
 	struct i40e_virtchnl_pf_event *pf_msg =
 			(struct i40e_virtchnl_pf_event *)msg;
+	struct i40e_vf *vf = I40EVF_DEV_PRIVATE_TO_VF(dev->data->dev_private);
 
 	switch (pf_msg->event) {
 	case I40E_VIRTCHNL_EVENT_RESET_IMPENDING:
@@ -1318,6 +1321,8 @@  i40evf_handle_pf_event(__rte_unused struct rte_eth_dev *dev,
 		break;
 	case I40E_VIRTCHNL_EVENT_LINK_CHANGE:
 		PMD_DRV_LOG(DEBUG, "VIRTCHNL_EVENT_LINK_CHANGE event\n");
+		vf->link_up = pf_msg->event_data.link_event.link_status;
+		vf->link_speed = pf_msg->event_data.link_event.link_speed;
 		break;
 	case I40E_VIRTCHNL_EVENT_PF_DRIVER_CLOSE:
 		PMD_DRV_LOG(DEBUG, "VIRTCHNL_EVENT_PF_DRIVER_CLOSE event\n");
@@ -2124,10 +2129,30 @@  i40evf_dev_link_update(struct rte_eth_dev *dev,
 	if (vf->version_major == I40E_DPDK_VERSION_MAJOR)
 		i40evf_get_link_status(dev, &new_link);
 	else {
-		/* Always assume it's up, for Linux driver PF host */
-		new_link.link_duplex = ETH_LINK_AUTONEG_DUPLEX;
-		new_link.link_speed  = ETH_LINK_SPEED_10000;
-		new_link.link_status = 1;
+		/* Linux driver PF host */
+		switch (vf->link_speed) {
+		case I40E_LINK_SPEED_100MB:
+			new_link.link_speed = ETH_LINK_SPEED_100;
+			break;
+		case I40E_LINK_SPEED_1GB:
+			new_link.link_speed = ETH_LINK_SPEED_1000;
+			break;
+		case I40E_LINK_SPEED_10GB:
+			new_link.link_speed = ETH_LINK_SPEED_10G;
+			break;
+		case I40E_LINK_SPEED_20GB:
+			new_link.link_speed = ETH_LINK_SPEED_20G;
+			break;
+		case I40E_LINK_SPEED_40GB:
+			new_link.link_speed = ETH_LINK_SPEED_40G;
+			break;
+		default:
+			new_link.link_speed = ETH_LINK_SPEED_100;
+			break;
+		}
+		/* full duplex only */
+		new_link.link_duplex = ETH_LINK_FULL_DUPLEX;
+		new_link.link_status = vf->link_up ? 1 : 0;
 	}
 	i40evf_dev_atomic_write_link_status(dev, &new_link);